#!/usr/local/bin/perl5
# bwnmz.pl - by furukawa@tcp-ip.or.jp
{
    my($tmp) = $0;
    $tmp = "./$tmp" if $tmp !~ /[\/\\]/;
    while ($tmp =~ /^(.*[\/\\])/){
        unshift(@INC, $1);
        last if !-l $tmp;
        $tmp = $1 . $tmp if ($tmp = readlink($tmp)) !~ /^\//;
    }
}

require 'pconfig.pl';
require 'db.pl';
require 'iocode.pl';
require 'output.pl';
require 'bw.pl';
require 'mb.pl';
require 'sb.pl';
require 'inttype.pl';

while (@ARGV && $ARGV[0] =~ s/^-//){
    $_ = shift;
    $quiet = 1 if /q/;
    $verbose = 1 if /v/;
    $do_s = 1 if /s/;
    $do_m = 1 if /m/;
    $Debug = 1 if /d/;
}
$| = 1;
$do_s = $do_m = 1 if !$do_s && !$do_m;

if (@ARGV){
    if (-d ($DbPath = shift)){
        $DbPath .= '/' unless $DbPath =~ /\/$/;
        $DbPath .= 'NMZ';
    }
}else{
    $DbPath = "NMZ";
}
&opendb($DbPath);
&set_inttype;

sub openbw{
    my $ext = shift;
    my $ret = (open(FH, ">$DbPath.$ext")
               && open(FHI, ">$DbPath.$ext" . 'i'));
    binmode(FH);
    binmode(FHI);
    if ($ext eq 's'){
        $Chars = \&sbchars;
        $Rev = \&sbrev;
        $Elem = $SbElem;
    }else{
        $Chars = \&mbchars;
        $Rev = \&mbrev;
        $Elem = $MbElem;
    }
    %DAT = ();
    %TBL = ();
    $ret;
}

sub closebw{
    my($offset, $hi, $lo, $str, $strh);
    &$Rev;
    for ($hi = 0; $hi < $Elem; $hi++){
        $strh = $Rev[$hi];
        for ($lo = 0; $lo < $Elem; $lo++){
            $str = $strh . $Rev[$lo];

            &output("$str: $TBL{$str}\n") if $TBL{$str} && $verbose;
            print FH $DAT{$str};
            print FHI pack($IntNType, $offset);

            if ($IntType eq 'w'){
                $offset += length($DAT{$str});
            }else{
                $offset += length($DAT{$str}) / $IntSize;
            }
        }
    }
    close(FH);
    close(FHI);
}

&openbw($ext = 's');
$x = 0;

while ($x < $DbNdx{'INDEXINDEX'}){
    $_ = &readindexindex($x);
    my $buf = $_;
    my $px = &packw($x);
    my($c1, $c2, %bigram);
    if ($verbose){
        &output("$_\n");
    }elsif (!$quiet and &progress_timer){
        &output("$x/$DbNdx{'INDEXINDEX'} $_\n");
    }

    s/[^0-9A-Za-z\x80-\xff]/_/g;

    if (/[\xa1-\xfe]/ && $ext eq 's'){
        &closebw;
        &openbw($ext = 'm');
    }

    while (($c1, $c2) = &$Chars($_)){
        $bigram{"$c1$c2"} = 1;
    }

    foreach $_ (sort keys %bigram){
        $DAT{$_} .= $px;
        $TBL{$_} .= "$buf " if $verbose;
    }
    $x++;
}
&closebw;

sub mbrev{
    @Rev = ();
    for (my $x = $MbByteL; $x <= $MbByteR; $x++){
        push(@Rev, chr($x));
    }
}

sub sbrev{
    @Rev = ();
    push(@Rev, '_');
    for (my $x = ord('a'); $x <= ord('z'); $x++){
        push(@Rev, chr($x));
    }
    for (my $x = 0; $x <= 9; $x++){
        push(@Rev, $x);
    }
}

sub packw{
    my $x = shift;
    
    if ($PackWSub){
        my $ret = chr($x & 0x7f);
        $ret = chr(0x80 | ($x & 0x7f)) . $ret while $x >>= 7;
        $ret;
    }else{
        pack($IntType, $x);
    }
}

sub debug_log{
    print shift if $Debug;
}

sub progress_timer{
    my $now = time;
    $progress_timer = $^T unless defined $progress_timer;
    return 0 if $now - $progress_timer < 5;
    $progress_timer = $now;
    return 1;
}
